==> com.liferay.portal.workflow.kaleo.runtime.impl-6.0.30-sources.jar: com/liferay/portal/workflow/kaleo/runtime/internal/messaging/KaleoWorkflowMessagingConfigurator.java <==
/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.workflow.kaleo.runtime.internal.messaging;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.messaging.Destination;
import com.liferay.portal.kernel.messaging.DestinationConfiguration;
import com.liferay.portal.kernel.messaging.DestinationFactory;
import com.liferay.portal.kernel.messaging.DestinationNames;
import com.liferay.portal.kernel.messaging.MessageBus;
import com.liferay.portal.kernel.messaging.MessageListener;
import com.liferay.portal.kernel.messaging.proxy.ProxyMessageListener;
import com.liferay.portal.kernel.scheduler.messaging.SchedulerEventMessageListenerWrapper;
import com.liferay.portal.kernel.util.HashMapDictionaryBuilder;
import com.liferay.portal.kernel.workflow.WorkflowDefinitionManager;
import com.liferay.portal.kernel.workflow.WorkflowEngineManager;
import com.liferay.portal.kernel.workflow.WorkflowInstanceManager;
import com.liferay.portal.kernel.workflow.WorkflowLogManager;
import com.liferay.portal.kernel.workflow.WorkflowStatusManager;
import com.liferay.portal.kernel.workflow.WorkflowTaskManager;
import com.liferay.portal.kernel.workflow.comparator.WorkflowComparatorFactory;
import com.liferay.portal.kernel.workflow.messaging.DefaultWorkflowDestinationEventListener;
import com.liferay.portal.workflow.kaleo.runtime.constants.KaleoRuntimeDestinationNames;
import com.liferay.portal.workflow.kaleo.runtime.internal.timer.messaging.TimerMessageListener;

import java.util.Dictionary;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;

/**
 * @author Michael C. Han
 */
@Component(immediate = true, service = KaleoWorkflowMessagingConfigurator.class)
public class KaleoWorkflowMessagingConfigurator {

	@Activate
	protected void activate(BundleContext bundleContext) {
		_bundleContext = bundleContext;

		_registerWorkflowDefinitionLinkDestination();

		_registerWorkflowMessageListeners();

		_registerWorkflowTimerDestination();

		_registerSchedulerEventMessageListener();
	}

	@Deactivate
	protected void deactivate() {
		_unregisterKaleoWorkflowDestinations();

		_unregisterWorkflowEngineDestinationListener();

		_unregisterWorkflowMessageListeners();

		_unregisterSchedulerEventMessageListener();

		_bundleContext = null;
	}

	private void _registerDestination(
		DestinationConfiguration destinationConfiguration) {

		Destination destination = _destinationFactory.createDestination(
			destinationConfiguration);

		Dictionary<String, Object> properties =
			HashMapDictionaryBuilder.<String, Object>put(
				"destination.name", destination.getName()
			).build();

		ServiceRegistration<Destination> serviceRegistration =
			_bundleContext.registerService(
				Destination.class, destination, properties);

		_serviceRegistrations.put(destination.getName(), serviceRegistration);
	}

	private MessageListener _registerProxyMessageListener(
		Object manager, String destinationName) {

		ProxyMessageListener proxyMessageListener = new ProxyMessageListener();

		proxyMessageListener.setManager(manager);
		proxyMessageListener.setMessageBus(_messageBus);

		_messageBus.registerMessageListener(
			destinationName, proxyMessageListener);

		_proxyMessageListeners.put(destinationName, proxyMessageListener);

		return proxyMessageListener;
	}

	private void _registerSchedulerEventMessageListener() {
		SchedulerEventMessageListenerWrapper
			schedulerEventMessageListenerWrapper =
				new SchedulerEventMessageListenerWrapper();

		schedulerEventMessageListenerWrapper.setMessageListener(
			_timerMessageListener);

		_schedulerEventMessageListenerServiceRegistration =
			_bundleContext.registerService(
				MessageListener.class, schedulerEventMessageListenerWrapper,
				HashMapDictionaryBuilder.<String, Object>put(
					"destination.name",
					KaleoRuntimeDestinationNames.WORKFLOW_TIMER
				).build());
	}

	private void _registerWorkflowDefinitionLinkDestination() {
		DestinationConfiguration destinationConfiguration =
			new DestinationConfiguration(
				DestinationConfiguration.DESTINATION_TYPE_SYNCHRONOUS,
				KaleoRuntimeDestinationNames.WORKFLOW_DEFINITION_LINK);

		_registerDestination(destinationConfiguration);
	}

	private void _registerWorkflowMessageListeners() {
		_defaultWorkflowDestinationEventListener =
			new DefaultWorkflowDestinationEventListener();

		_defaultWorkflowDestinationEventListener.setWorkflowEngineName(
			"Liferay Kaleo Workflow Engine");

		MessageListener workflowComparatorMessageListener =
			_registerProxyMessageListener(
				_workflowComparatorFactory,
				DestinationNames.WORKFLOW_COMPARATOR);

		_defaultWorkflowDestinationEventListener.
			setWorkflowComparatorFactoryListener(
				workflowComparatorMessageListener);

		MessageListener workflowDefinitionManagerProxyMessageListener =
			_registerProxyMessageListener(
				_workflowDefinitionManager,
				DestinationNames.WORKFLOW_DEFINITION);

		_defaultWorkflowDestinationEventListener.
			setWorkflowDefinitionManagerListener(
				workflowDefinitionManagerProxyMessageListener);

		MessageListener workflowEngineManagerProxyMessageListener =
			_registerProxyMessageListener(
				_workflowEngineManager, DestinationNames.WORKFLOW_ENGINE);

		_defaultWorkflowDestinationEventListener.
			setWorkflowEngineManagerListener(
				workflowEngineManagerProxyMessageListener);

		MessageListener workflowInstanceManagerProxyMessageListener =
			_registerProxyMessageListener(
				_workflowInstanceManager, DestinationNames.WORKFLOW_INSTANCE);

		_defaultWorkflowDestinationEventListener.
			setWorkflowInstanceManagerListener(
				workflowInstanceManagerProxyMessageListener);

		MessageListener workflowLogManagerProxyMessageListener =
			_registerProxyMessageListener(
				_workflowLogManager, DestinationNames.WORKFLOW_LOG);

		_defaultWorkflowDestinationEventListener.setWorkflowLogManagerListener(
			workflowLogManagerProxyMessageListener);

		MessageListener workflowStatusManagerProxyMessageListener =
			_registerProxyMessageListener(
				_workflowStatusManager, DestinationNames.WORKFLOW_STATUS);

		_defaultWorkflowDestinationEventListener.
			setWorkflowStatusManagerListener(
				workflowStatusManagerProxyMessageListener);

		MessageListener workflowTaskManagerProxyMessageListener =
			_registerProxyMessageListener(
				_workflowTaskManager, DestinationNames.WORKFLOW_TASK);

		_defaultWorkflowDestinationEventListener.setWorkflowTaskManagerListener(
			workflowTaskManagerProxyMessageListener);

		_workflowEngineDestination.addDestinationEventListener(
			_defaultWorkflowDestinationEventListener);
	}

	private void _registerWorkflowTimerDestination() {
		DestinationConfiguration destinationConfiguration =
			new DestinationConfiguration(
				DestinationConfiguration.DESTINATION_TYPE_PARALLEL,
				KaleoRuntimeDestinationNames.WORKFLOW_TIMER);

		destinationConfiguration.setMaximumQueueSize(_MAXIMUM_QUEUE_SIZE);

		RejectedExecutionHandler rejectedExecutionHandler =
			new ThreadPoolExecutor.CallerRunsPolicy() {

				@Override
				public void rejectedExecution(
					Runnable runnable, ThreadPoolExecutor threadPoolExecutor) {

					if (_log.isWarnEnabled()) {
						_log.warn(
							"The current thread will handle the request " +
								"because the workflow timer task queue is at " +
									"its maximum capacity");
					}

					super.rejectedExecution(runnable, threadPoolExecutor);
				}

			};

		destinationConfiguration.setRejectedExecutionHandler(
			rejectedExecutionHandler);

		_registerDestination(destinationConfiguration);
	}

	private void _unregisterKaleoWorkflowDestinations() {
		for (ServiceRegistration<Destination> serviceRegistration :
				_serviceRegistrations.values()) {

			Destination destination = _bundleContext.getService(
				serviceRegistration.getReference());

			serviceRegistration.unregister();

			destination.destroy();
		}

		_serviceRegistrations.clear();
	}

	private void _unregisterSchedulerEventMessageListener() {
		if (_schedulerEventMessageListenerServiceRegistration == null) {
			return;
		}

		_schedulerEventMessageListenerServiceRegistration.unregister();

		_schedulerEventMessageListenerServiceRegistration = null;
	}

	private void _unregisterWorkflowEngineDestinationListener() {
		_workflowEngineDestination.removeDestinationEventListener(
			_defaultWorkflowDestinationEventListener);
	}

	private void _unregisterWorkflowMessageListeners() {
		for (Map.Entry<String, MessageListener> entry :
				_proxyMessageListeners.entrySet()) {

			_messageBus.unregisterMessageListener(
				entry.getKey(), entry.getValue());
		}

		_proxyMessageListeners.clear();
	}

	private static final int _MAXIMUM_QUEUE_SIZE = 200;

	private static final Log _log = LogFactoryUtil.getLog(
		KaleoWorkflowMessagingConfigurator.class);

	private BundleContext _bundleContext;
	private DefaultWorkflowDestinationEventListener
		_defaultWorkflowDestinationEventListener;

	@Reference
	private DestinationFactory _destinationFactory;

	@Reference
	private MessageBus _messageBus;

	private final Map<String, MessageListener> _proxyMessageListeners =
		new HashMap<>();
	private ServiceRegistration<MessageListener>
		_schedulerEventMessageListenerServiceRegistration;
	private final Map<String, ServiceRegistration<Destination>>
		_serviceRegistrations = new HashMap<>();

	@Reference
	private TimerMessageListener _timerMessageListener;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowComparatorFactory _workflowComparatorFactory;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowDefinitionManager _workflowDefinitionManager;

	@Reference(
		target = "(destination.name=" + DestinationNames.WORKFLOW_ENGINE + ")"
	)
	private Destination _workflowEngineDestination;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowEngineManager _workflowEngineManager;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowInstanceManager _workflowInstanceManager;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowLogManager _workflowLogManager;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowStatusManager _workflowStatusManager;

	@Reference(target = "(proxy.bean=false)")
	private WorkflowTaskManager _workflowTaskManager;

}

==> com.liferay.portal.workflow.kaleo.runtime.impl-6.0.49-sources.jar: com/liferay/portal/workflow/kaleo/runtime/internal/messaging/KaleoWorkflowMessagingConfigurator.java <==
/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 */

package com.liferay.portal.workflow.kaleo.runtime.internal.messaging;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.messaging.Destination;
import com.liferay.portal.kernel.messaging.DestinationConfiguration;
import com.liferay.portal.kernel.messaging.DestinationFactory;
import com.liferay.portal.kernel.util.MapUtil;
import com.liferay.portal.workflow.kaleo.runtime.constants.KaleoRuntimeDestinationNames;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.RejectedExecutionHandler;
import java.util.concurrent.ThreadPoolExecutor;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;

/**
 * @author Michael C. Han
 */
@Component(service = KaleoWorkflowMessagingConfigurator.class)
public class KaleoWorkflowMessagingConfigurator {

	@Activate
	protected void activate(BundleContext bundleContext) {
		_bundleContext = bundleContext;

		_registerWorkflowDefinitionLinkDestination();

		_registerWorkflowTimerDestination();
	}

	@Deactivate
	protected void deactivate() {
		for (ServiceRegistration<Destination> serviceRegistration :
				_serviceRegistrations) {

			serviceRegistration.unregister();
		}
	}

	private void _registerDestination(
		DestinationConfiguration destinationConfiguration) {

		Destination destination = _destinationFactory.createDestination(
			destinationConfiguration);

		_serviceRegistrations.add(
			_bundleContext.registerService(
				Destination.class, destination,
				MapUtil.singletonDictionary(
					"destination.name", destination.getName())));
	}

	private void _registerWorkflowDefinitionLinkDestination() {
		DestinationConfiguration destinationConfiguration =
			new DestinationConfiguration(
				DestinationConfiguration.DESTINATION_TYPE_SYNCHRONOUS,
				KaleoRuntimeDestinationNames.WORKFLOW_DEFINITION_LINK);

		_registerDestination(destinationConfiguration);
	}

	private void _registerWorkflowTimerDestination() {
		DestinationConfiguration destinationConfiguration =
			new DestinationConfiguration(
				DestinationConfiguration.DESTINATION_TYPE_PARALLEL,
				KaleoRuntimeDestinationNames.WORKFLOW_TIMER);

		destinationConfiguration.setMaximumQueueSize(_MAXIMUM_QUEUE_SIZE);

		RejectedExecutionHandler rejectedExecutionHandler =
			new ThreadPoolExecutor.CallerRunsPolicy() {

				@Override
				public void rejectedExecution(
					Runnable runnable, ThreadPoolExecutor threadPoolExecutor) {

					if (_log.isWarnEnabled()) {
						_log.warn(
							"The current thread will handle the request " +
								"because the workflow timer task queue is at " +
									"its maximum capacity");
					}

					super.rejectedExecution(runnable, threadPoolExecutor);
				}

			};

		destinationConfiguration.setRejectedExecutionHandler(
			rejectedExecutionHandler);

		_registerDestination(destinationConfiguration);
	}

	private static final int _MAXIMUM_QUEUE_SIZE = 200;

	private static final Log _log = LogFactoryUtil.getLog(
		KaleoWorkflowMessagingConfigurator.class);

	private BundleContext _bundleContext;

	@Reference
	private DestinationFactory _destinationFactory;

	private final List<ServiceRegistration<Destination>> _serviceRegistrations =
		new ArrayList<>();

}
